#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require 'mxx_ru/ex'

require 'mxx_ru/util'

module Mxx_ru

	#    .
	BUILD_ROOT = "build.rb"

	#   .
	class	Target_state
		#       .
		ABSENT = 0
		#   .
		REBUILT = 1
		#  ,     
		#   .
		EXISTS = 2

		attr_reader	:state

		def initialize( a_state = ABSENT )
			@state = a_state
		end

		#   .
		#    Target_state.
		def Target_state.detect(
			#  .
			name,
			#  . Array  String.
			depends )

			state = ABSENT

			#   dry-run    ,  
			# .
			if !Mxx_ru::Util::Mode.instance.is_dry_run
				if FileTest.exists?( name )
					#        .
					target_mtime = File.mtime( name )
					if nil != depends.find { |d|
							if !FileTest.exists?( d )
								raise Mxx_ru::Missing_depend_file_ex.new( d, name )
							end
							depend_mtime = File.mtime( d )
							target_mtime < depend_mtime
						}
						#    .
						state = REBUILT
					else
						#       .
						state = EXISTS
					end
				end
			end

			return Target_state.new( state )
		end

	end # Class Target_state

	#   .
	#     --     .
	@@defined_targets = Hash.new

	#  ,     .
	#     setup_target    .
	@@first_target_alias = nil

	def	Mxx_ru.try_set_first_target_alias( a_alias )
		if nil == @@first_target_alias
			@@first_target_alias = a_alias.clone
		end
	end

	def Mxx_ru.setup_target( target, build_enabled = true )
		begin
			if !( target_defined_for?( target.prj_alias ) )
				@@defined_targets[ target.prj_alias ] = target;
			end

			if build_enabled && nil != @@first_target_alias &&
				@@first_target_alias == target.prj_alias
				#    ,     .

				#        ,
				#     ,   
				#        
				#   .
				if BUILD_ROOT != target.prj_alias
					if FileTest.exists?( BUILD_ROOT )
						require BUILD_ROOT
					end
				end

				if Mxx_ru::Util::Mode.instance.is_clean
					target.clean
				else
					Mxx_ru::Util::build_call_wrapper( target )
				end
			end

			return target
		rescue Mxx_ru::Ex => ex
			$stderr.print "<<<[#{ex.class.name}]\t#{ex}>>>\n"
			exit( -2 )
		end
	end

	def Mxx_ru.query_target( prj_alias, mandatory )
		target = @@defined_targets[ prj_alias ]

		#    ,      .
		#      .
		if target.nil? && mandatory
			raise Target_unknown_ex.new( prj_alias )
		end

		return target
	end

	def Mxx_ru.target_defined_for?(prj_alias)
		return true if @@defined_targets.has_key?( prj_alias )
		return false
	end

	#       .
	class	Abstract_generator
		#        .
		#      .
		def	build(
			# ,   .
			target )
			raise Abstract_method_ex.new( "Mxx_ru::Abstract_generator::build" )
		end

		#        .
		def	clean(
			# ,   .
			target )
			raise Abstract_method_ex.new( "Mxx_ru::Abstract_generator::clean" )
		end
	end

	#     .
	class	Abstract_target
		#    ,    
		#  .
		#    ,     
		# . ,    composite  unittest.
		attr_reader	:mxx_full_targets_names

		#   .
		attr_reader	:mxx_required_prjs

		#    .
		attr_reader	:mxx_generators

		def	initialize( a_prj_alias )
			@mxx_full_targets_names = Array.new
			@mxx_prj_alias = a_prj_alias
			@mxx_required_prjs = Array.new
			@mxx_generators = Array.new

			Mxx_ru.try_set_first_target_alias( a_prj_alias )
		end

		#    ,    
		#  .
		def	mxx_add_full_target_name( a_full_name )
			@mxx_full_targets_names << a_full_name
		end

		def	prj_alias
			return @mxx_prj_alias
		end

		#        .
		#
		#  -,     .
		def required_prj( a_prj )
			#     ,   . 
			#      
			#     target.   target
			#   ,    .
			if !Mxx_ru::target_defined_for?( a_prj )
				require a_prj
			end

			target = Mxx_ru::query_target( a_prj, true )
			@mxx_required_prjs << target

			return target
		end

		#      .
		#    -.
		def	generator( a_generator )
			@mxx_generators << a_generator
			return a_generator
		end

		#      .
		#    Target_state.
		def	build
			raise Abstract_method_ex.new( "Abstract_target::build" )
		end

		#      .
		def	clean
			raise Abstract_method_ex.new( "Abstract_target::clean" )
		end

		#      .
		def	Abstract_target.run(
			#   Array  String
			a_cmd_lines,
			# ,    ,    
			#     .
			a_to_destroy_on_fail )

			a_cmd_lines.each { |c|
					if Mxx_ru::Util::Mode.instance.is_show_cmd
						puts "<<< #{c} >>>"
					end

					#   dry-run    .
					if !Mxx_ru::Util::Mode.instance.is_dry_run
						if !system( c )
							a_to_destroy_on_fail.each { |d|
								Mxx_ru::Util::delete_file( d ) }
							raise Build_ex.new( c, $? )
						end
					end
				}
		end

	protected
		#   ,     .
		#  Mxx_ru::Target_state.
		def	build_required_projects
			state = Mxx_ru::Target_state::EXISTS
			@mxx_required_prjs.each { |p|
					prj_state = Mxx_ru::Util::build_call_wrapper( p )
					if Mxx_ru::Target_state::EXISTS != prj_state.state
						state = prj_state.state
					end
				}

			return Mxx_ru::Target_state.new( state )
		end

		#   ,     .
		#  ,     dry-run.
		def	clean_required_prjs
			if !Mxx_ru::Util::Mode.instance.is_dry_run
				@mxx_required_prjs.each { |p| p.clean }
			end
		end

	end

end
