#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require 'singleton'

module	Mxx_ru
	module	Cpp

		module	Analyzer
			# ,    .
			class	Source_file
				#  ,    .
				attr_reader	:is_processed
				#  -.
				# Hash,   String,  .
				attr_reader	:depends

				def initialize
					@is_processed = false
					@depends = Hash.new
				end

				#   ,   .
				def	processed
					@is_processed = true
				end

				#   .
				def	<<( a_name )
					@depends[ a_name ]= nil
				end

			end # class Source_file

			class	Cpp_analyzer
				include Singleton

				@@include_regex = Regexp.new(
					'^[ \t]*#[ \t]*include[ \t]*(<|")([^>"]+)(>|")' )

				def	initialize
					#   String,  -- Source_file.
					@sources = Hash.new
					#  ,      .
					@include_paths = Array.new
				end

				#    .
				def	add_sources(
					#   Array  String.
					a_sources )

					a_sources.each { |s|
							if !@sources.key?( s )
								@sources[ s ] = Source_file.new
							end
						}
				end

				#     .
				def	add_include_paths(
					#   Array  String.
					a_paths )

					a_paths.each { |p|
							if !@include_paths.include?( p )
								@include_paths << p
							end
						}
				end

				#   .
				def	analyze
					#       ,  
					#      .
					#     ,   
					#    .
					old_size = 0
					files = @sources.keys
					while old_size != files.size
						files.each { |file_name|
								current = @sources[ file_name ]
								if !current.is_processed
									process_file( file_name, current )
								end
							}

						old_size = files.size
						files = @sources.keys
					end
				end

				#      .
				#  Array  String.
				def	get_depends_for( a_file_name )
					result_hash = Hash.new
					file_obj = @sources[ a_file_name ]
					if nil != file_obj
						to_review = file_obj.depends
						to_review_next = Hash.new

						while to_review.size != to_review_next.size
							to_review.each_key { |f|
									if !result_hash.key?( f )
										result_hash[ f ] = nil
										to_review_next_obj = @sources[ f ]
										to_review_next_obj.depends.each_key { |k|
												to_review_next[ k ] = nil }
									end
								}

							to_review = to_review_next
							to_review_next = Hash.new
						end
					end

					return result_hash.keys
				end

			protected
				#   .
				def	process_file(
					#  .
					a_file_name,
					#   .
					a_file_obj )

					#       include.
					std_path_included = Array.new
					local_path_included = Array.new
					IO.foreach( a_file_name ) { |line|
							r = @@include_regex.match( line )
							if nil != r
								if "<" == r[ 1 ]
									std_path_included << r[ 2 ]
								else
									local_path_included << r[ 2 ]
								end
							end
						}

					#  ,     include <>
					#       .
					std_path_included.each { |f|
							process_std_include_path_file( f, a_file_obj )
						}

					if local_path_included.size
						# ,     include ""
						#      
						#   .
						local_path = File.dirname( a_file_name )
						local_path_included.each { |f|
								process_local_include_path_file( f, local_path, a_file_obj )
							}
					end

					a_file_obj.processed
				end

				#  ,     include <>
				def	process_std_include_path_file(
					#  .
					a_file_name,
					#  ,     include.
					a_owner_obj )

					@include_paths.find { |path|
							test_name = File.join( [ path, a_file_name ] )
							if File.exists?( test_name )
								add_depend_to( a_owner_obj, test_name )
								true
							else
								false
							end
						}
				end

				#  ,     include ""
				def	process_local_include_path_file(
					#  .
					a_file_name,
					# ,    owner_obj.
					a_local_path,
					#  ,     include.
					a_owner_obj )

					test_name = File.join( [ a_local_path, a_file_name ] )
					if File.exists?( test_name )
						add_depend_to( a_owner_obj, test_name )
					else
						process_std_include_path_file( a_file_name, a_owner_obj )
					end
				end

				#    .
				def	add_depend_to(
					#  .
					a_owner_obj,
					#   .
					a_depend_name )

					a_owner_obj << a_depend_name

					if !@sources.key?( a_depend_name )
						#     .
						@sources[ a_depend_name ] = Source_file.new
					end
				end
			end # class Cpp_analyzer

		end # module Analyzer
	end # module cpp
end # module Mxx_ru
