#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require 'mxx_ru/cpp/toolset'

module Mxx_ru
	module Cpp

		#   toolset    
		#  .
		#     toolset-.
		#    MXX_RU_CPP_TOOLSET.
		#    :
		# <file> [tag=value [tag=value [...]]
		#  <file>    require.
		#      toolset  
		#   -toolset.
		#  -toolset  ,     
		# tag=value       toolset 
		#  setup_tag.
		def	Cpp.detect_toolset
			begin
				env_var = "MXX_RU_CPP_TOOLSET"

				env = ENV[ env_var ]
				if nil == env
					raise Mxx_ru::Invalid_value_ex.new(
						"Environment variable '#{env_var}' not found (not set " +
						"or empty" )
				end

				#   ,    -toolset.
				toolset_file_name_regexp = /^(\S+)(.*)$/
				file_name_match = toolset_file_name_regexp.match( env )
				if nil == file_name_match
					raise Mxx_ru::Invalid_value_ex.new(
						"Invalid format of environment variable '#{env_var}'. " +
						"Expected format: #{env_var}=<file>[ <tag>=<value>[ " +
						"<tag>=<value>[ ...]]]" )
				end

				file_name = file_name_match[ 1 ]
				unparsed_tags = file_name_match[ 2]

				#    .
				require file_name
				if nil == toolset()
					raise Mxx_ru::Invalid_value_ex.new(
						"C/C++ Toolset is not set by file '#{file_name}'" )
				end

				#       toolset-.
				setup_toolset_tags( toolset(), unparsed_tags )

			rescue Mxx_ru::Ex => ex
				$stderr.print "<<<[#{ex.class.name}]\t#{ex}>>>\n"
				exit( -1 )
			end
		end

		def	Cpp.setup_toolset_tags( a_toolset,
			#   <tag>=<value>[ <tag>=<value> [<tag>=<value> ...]]
			a_unparsed_tags )

			a_unparsed_tags.scan( /(\S+)=(\S+)/ ) { |t, v|
				a_toolset.setup_tag( t, v )
			}

		end

	end # module Cpp
end # module Mxx_ru
