#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require 'mxx_ru/ex'

module	Mxx_ru

	module Cpp

		#        
		#      .
		class	Obj_placement
			#     ,  
			#    (.obj  .o) .
			#
			# _source_path_name_  ,     .
			#
			# _toolset_  toolset.
			#
			# _target_	 .
			def	get_obj(
				source_path_name,
				toolset,
				target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Obj_placement::get_obj" )
			end

			#     ,  
			#     mswin-.
			#
			# _source_path_name_  ,     .
			#
			# _toolset_  toolset.
			#
			# _target_	 .
			def	get_mswin_res(
				source_path_name,
				toolset,
				target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Obj_placement::get_mswin_res" )
			end

			#     ,  
			#    .
			#
			# _source_path_name_  ,     .
			#
			# _toolset_  toolset.
			#
			# _target_	 .
			def	get_lib(
				source_path_name,
				toolset,
				target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Obj_placement::get_lib" )
			end

			#     ,  
			#   - .
			#
			# _source_path_name_  ,     .
			#
			# _toolset_  toolset.
			#
			# _target_	 .
			def	get_dll(
				source_path_name,
				toolset,
				target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Obj_placement::get_dll" )
			end

			#     ,  
			#   EXE-.
			#
			# _source_path_name_  ,     .
			#
			# _toolset_  toolset.
			#
			# _target_	 .
			def	get_exe(
				source_path_name,
				toolset,
				target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Obj_placement::get_exe" )
			end
		end # class Obj_placement

		#      ,  
		#         .
		#   lib, dll, exe   --  .
		#  get_obj  get_mswin_res ,  
		#  .    ,  
		#   .
		class	Source_subdir_obj_placement < Obj_placement
			attr	:subdir

			def initialize( a_subdir = "o" )
				#  ,      .
				@subdir = a_subdir.clone
			end

			def	get_obj(
				source_path_name,
				toolset,
				target )

				result = File.join( source_path_name, subdir )
				Mxx_ru::Util.ensure_path_exists( result )

				return result
			end

			def	get_mswin_res(
				source_path_name,
				toolset,
				target )

				result = File.join( source_path_name, subdir )
				Mxx_ru::Util.ensure_path_exists( result )

				return result
			end

			def	get_lib(
				source_path_name,
				toolset,
				target )

				return String.new( source_path_name )
			end

			def	get_dll(
				source_path_name,
				toolset,
				target )

				return String.new( source_path_name )
			end

			def	get_exe(
				source_path_name,
				toolset,
				target )

				return String.new( source_path_name )
			end

		end # class Source_subdir_obj_placement

		#      , 
		#      ,
		#       runtime.
		#
		# ,    src/lib/l.cpp, src/main/m.cpp, 
		#    lib/l.lib    m.exe.
		#      RELEASE,   
		# : release/src/lib/l.obj, release/src/main/m.obj,
		# release/lib/l.lib  release/m.exe.
		#      
		#  (, release/src/lib, release/src/main,...). 
		#  ,    .
		#
		#  :
		#
		#class	Build < Mxx_ru::Cpp::Composite_target
		#	def	initialize( a_alias = Mxx_ru::BUILD_ROOT )
		#		global_obj_placement(
		#			Mxx_ru::Cpp::Runtime_subdir_obj_placement.new(
		#				"output" ) )
		#
		#		required_prj( "src/lib/prj.rb" )
		#		required_prj( "src/main/prj.rb" )
		#	end
		#end
		#
		class	Runtime_subdir_obj_placement < Obj_placement
			#  ,     
			#   runtime-.
			attr_reader	:root_dir
			#  ,     root_dir 
			#  Mxx_ru::Cpp::RUNTIME_DEBUG.
			attr_reader	:debug_subdir
			#  ,     root_dir 
			#  Mxx_ru::Cpp::RUNTIME_DEFAULT.
			attr_reader	:default_subdir
			#  ,     root_dir 
			#  Mxx_ru::Cpp::RUNTIME_RELEASE.
			attr_reader	:release_subdir

			# a_root_dir	,     
			#    runtime.   nil, 
			#      
			#   .
			#
			# _a_debug_subdir_     Mxx_ru::Cpp::RUNTIME_DEBUG.
			#
			# _a_default_subdir_     Mxx_ru::Cpp::RUNTIME_DEFAULT.
			#
			# _a_release_subdir_     Mxx_ru::Cpp::RUNTIME_RELEASE.
			def initialize(
				a_root_dir = nil,
				a_debug_subdir = "debug",
				a_default_subdir = "default",
				a_release_subdir = "release" )

				if a_root_dir
					@root_dir = a_root_dir
				else
					@root_dir = "./"
				end

				@debug_subdir = a_debug_subdir
				@default_subdir = a_default_subdir
				@release_subdir = a_release_subdir
			end

			#  ,   - .
			#    get_*    .
			def	get_obj(
				source_path_name,
				toolset,
				target )

				if source_path_name &&
					"" != source_path_name &&
					"." != source_path_name
					result = File.join( @root_dir, runtime_mode_path( target ),
						source_path_name )
				else
					result = File.join( @root_dir, runtime_mode_path( target ) )
				end

				Mxx_ru::Util.ensure_path_exists( result )

				return result
			end

			#     get_obj.
			def	get_mswin_res(
				source_path_name,
				toolset,
				target )

				return get_obj( source_path_name, toolset, target )
			end

			#     get_obj.
			def	get_lib(
				source_path_name,
				toolset,
				target )

				return get_obj( source_path_name, toolset, target )
			end

			#     get_obj.
			def	get_dll(
				source_path_name,
				toolset,
				target )

				return get_obj( source_path_name, toolset, target )
			end

			#     get_obj.
			def	get_exe(
				source_path_name,
				toolset,
				target )

				return get_obj( source_path_name, toolset, target )
			end

		protected
			#   ,   
			#  runtime- .
			#
			# _a_target_ ,    .
			def	runtime_mode_path( a_target )
				case a_target.mxx_runtime_mode
					when Mxx_ru::Cpp::RUNTIME_DEBUG
						return @debug_subdir

					when Mxx_ru::Cpp::RUNTIME_RELEASE
						return @release_subdir

					else
						return @default_subdir
				end
			end

		end # class Runtime_subdir_obj_placement

		#   obj_placement,   
		# -.
		def	Cpp.default_obj_placement
			return Source_subdir_obj_placement.new
		end

	end # module Cpp

end # module Mxx_ru
