#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require 'mxx_ru/cpp'

module Mxx_ru
module Cpp

	#     Qt.
	#
	#  :
	#
	# - moc-   moc-.
	#   cpp-  moc-
	# (, a.cpp -> a.moc)  hpp-  moc-
	# (, b.hpp -> moc_b.cpp);
	#
	# - hpp-, cpp-  ui-  
	# uic-.    
	# hpp-   moc-.
	# ,  c.ui  : .hpp, c.cpp,
	# moc_c.cpp.
	#
	#    cpp-
	# (   moc-,    
	# uic-)   cpp_source- .
	#
	#    defines,   
	#   defines  .
	#
	#         target,
	#   target   defines, 
	#   -.   defines 
	#   ,     
	#   :
	#
	#	generator( Mxx_ru::Cpp::Qt_gen.new( self, [ "QT_THREAD_SUPPORT" ] ) )
	#
	class	Qt_gen < Mxx_ru::Abstract_generator
		#  defines,    -.
		# QT_DLL, QT_THREAD_SUPPORT
		DEFAULT_DEFINES = [ "QT_DLL", "QT_THREAD_SUPPORT" ]

		#  moc-.
		# -: moc
		attr_accessor	:moc_name 

		#  uic-.
		# -: uic.
		attr_accessor	:uic_name

		#    cpp-.
		#     moc-,  
		#   uic-.
		# -: .cpp
		attr_accessor	:cpp_ext

		#    hpp-.
		#    uic-.
		# -: .hpp
		attr_accessor	:hpp_ext

		#    moc-.
		#    moc-.
		# -: .moc
		attr_accessor	:moc_ext

		#  ,     
		#   moc- .
		#    nil,   moc- 
		#     ,    .
		# -: nil (..  src/b.cpp   src/b.moc).
		attr_accessor	:moc_result_subdir

		#      hpp  moc.
		#
		#  : a.hpp -> moc_a.cpp.  moc_a.cpp 
		#  cpp_source  .
		attr_reader	:qt_h2moc_files

		#      cpp  moc.
		#
		#  : a.cpp -> a.moc.
		attr_reader	:qt_cpp2moc_files

		#      ui  hpp, cpp, moc.
		#
		#  : a.ui -> a.hpp, a.cpp, moc_a.cpp. 
		# a.cpp, moc_a.cpp   cpp_source  .
		attr_reader	:qt_ui_files

		# ,    .
		attr	:target

		# .
		def initialize( a_target, a_defines = DEFAULT_DEFINES )
			@moc_name = "moc"
			@uic_name = "uic"
			@cpp_ext = ".cpp"
			@hpp_ext = ".hpp"
			@moc_ext = ".moc"
			@moc_result_subdir = nil

			@qt_h2moc_files = Array.new
			@qt_cpp2moc_files = Array.new
			@qt_ui_files = Array.new

			defines_to_set = a_defines.flatten
			defines_to_set.each { |d| a_target.define( d ) }

			@target = a_target
		end

		#      hpp  moc.
		def	h2moc( a_file )
			@qt_h2moc_files << @target.create_full_src_file_name( a_file )
		end

		#      cpp  moc.
		def	cpp2moc( a_file )
			@qt_cpp2moc_files << @target.create_full_src_file_name( a_file )
		end

		#      ui  hpp, cpp, moc.
		def	ui( a_file )
			@qt_ui_files << @target.create_full_src_file_name( a_file )
		end

		#   .
		def	build( a_target )
			build_from_ui( a_target )
			build_from_h( a_target )
			build_from_cpp( a_target )
		end

		#     .
		def	clean( a_target )
			clean_from_ui( a_target )
			clean_from_h( a_target )
			clean_from_cpp( a_target )
		end

	protected

		#     ui-.
		def	build_from_ui( a_target )
			@qt_ui_files.each { |ui|
				files_from_ui = names_from_ui( ui, a_target )

				#    .
				header_file = files_from_ui[ 0 ]
				cpp_file = files_from_ui[ 1 ]

				@qt_h2moc_files << header_file 
				add_cpp_source( a_target, cpp_file )

				# ,    ,  
				#  .
				if Target_state::EXISTS != Target_state.detect(
					header_file, [ ui ] ).state
					Mxx_ru::Abstract_target::run(
						[ "#{@uic_name} -o #{header_file} #{ui}" ],
						[ header_file ] )
				end
				if Target_state::EXISTS != Target_state.detect(
					cpp_file, [ ui, header_file ] ).state
					Mxx_ru::Abstract_target::run(
						[ "#{@uic_name} -i #{header_file} " +
							"-o #{cpp_file} #{ui}" ],
						[ cpp_file ] )
				end
			}
		end

		#  ,     ui-.
		def	clean_from_ui( a_target )
			@qt_ui_files.each { |ui|
				files_from_ui = names_from_ui( ui, a_target )

				files_from_ui.each { |f| Mxx_ru::Util::delete_file( f ) }

				#   ,     .
				@qt_h2moc_files << files_from_ui[ 0 ]
				add_cpp_source( a_target, files_from_ui[ 1 ] )
			}
		end

		#   ,   uic-.
		#
		#     .   -- 
		#  hpp-.   --   cpp-.
		def	names_from_ui( a_ui_file, a_target )
			full_ui_name = Mxx_ru::Util::remove_file_ext( a_ui_file )
			result = [
				full_ui_name + @hpp_ext,
				full_ui_name + @cpp_ext
			]

			return result
		end

		#   hpp-  moc-.
		def	build_from_h( a_target )
			@qt_h2moc_files.each { |h|
				moc = moc_from_h( h, a_target )

				moc_full = moc
				header_full = h

				if Target_state::EXISTS != Target_state.detect(
					moc_full, [ header_full ] ).state
					Mxx_ru::Abstract_target::run(
						[ "#{moc_name} -o #{moc_full} " + 
							"#{header_full}" ],
						[ moc_full ] )
				end

				add_cpp_source( a_target, moc )
			}
		end

		#    hpp-  moc-.
		def	clean_from_h( a_target )
			@qt_h2moc_files.each { |h|
				moc = moc_from_h( h, a_target )

				Mxx_ru::Util::delete_file( moc )

				add_cpp_source( a_target, moc )
			}
		end

		#   ,    hpp-
		# moc-.
		def	moc_from_h( a_h_file, a_target )
			path = File.dirname( a_h_file )

			#  path == ".",  qt  
			#  include.
			path = "" if path == "./"

			if @moc_result_subdir
				if path.length > 0
					path = File.join( path, @moc_result_subdir )
				else
					path = @moc_result_subdir
				end
			end

			if path.length > 0
				#    .
				Mxx_ru::Util::ensure_path_exists( path )
			end

			r = File.join( path, "moc_" + File.basename(
				Mxx_ru::Util::remove_file_ext( a_h_file ) ) + @cpp_ext )

			return r
		end

		#   cpp-  moc-.
		def	build_from_cpp( a_target )
			@qt_cpp2moc_files.each { |cpp|
				moc = moc_from_cpp( cpp, a_target )

				moc_full = moc
				cpp_full = cpp

				if Target_state::EXISTS != Target_state.detect(
					moc_full, [ cpp_full ] ).state
					Mxx_ru::Abstract_target::run(
						[ "#{moc_name} -o #{moc_full} " + 
							"#{cpp_full}" ],
						[ moc_full ] )
				end
			}
		end

		#    cpp- moc-.
		def	clean_from_cpp( a_target )
			@qt_cpp2moc_files.each { |cpp|
				moc = moc_from_cpp( cpp, a_target )

				Mxx_ru::Util::delete_file( moc )
			}
		end

		#   ,    cpp-
		# moc-.
		def	moc_from_cpp( a_cpp_file, a_target )
			path = File.dirname( a_cpp_file )

			if @moc_result_subdir
				if path.length > 0
					path = File.join( path, @moc_result_subdir )
				else
					path = @moc_result_subdir
				end
			end

			if path.length > 0
				#    .
				Mxx_ru::Util::ensure_path_exists( path )
			end

			r = File.join( path, File.basename(
				Mxx_ru::Util::remove_file_ext( a_cpp_file ) ) + @moc_ext )

			return r
		end

		#   C++    
		#  sources_root.
		def	add_cpp_source( a_target, a_file )
			old_root = a_target.sources_root( "" )
			a_target.cpp_source( a_file )
			a_target.sources_root( old_root )
		end

	end # class Qt_gen

end
end

