#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require 'mxx_ru/ex'
require 'mxx_ru/util'

module	Mxx_ru
	module	Cpp

		# ,   .
		class	Obj_info
			#   .
			attr_reader	:name
			#   .
			attr_reader	:source
			#     
			#   .
			# Array  String.
			attr_reader	:options

			def initialize( a_name, a_source, a_options )
				@name = a_name
				@source = a_source
				@options = a_options
			end
		end # class Obj_info

		# ,  mswin-res-.
		class	Mswin_res_info
			attr_reader	:file_name
			attr_reader	:path

			def initialize( a_file_name, a_path )
				#   mswin-res-  .
				@file_name = a_file_name
				# ,     mswin-res-.
				@path = a_path
			end
		end # class Mswin_res_info

		# ,  .
		class	Lib_info
			attr_reader	:file_name
			attr_reader	:path
			attr_reader	:link_name

			def initialize( a_file_name, a_path, a_link_name )
				#     .
				@file_name = a_file_name
				# ,     .
				@path = a_path
				# ,    .
				@link_name = a_link_name
			end
		end # class Lib_info

		# ,  EXE-.
		class	Exe_info
			attr_reader	:file_name
			attr_reader	:path

			def initialize( a_file_name, a_path )
				#   EXE-  .
				@file_name = a_file_name
				# ,     EXE-.
				@path = a_path
			end
		end # class Exe_info

		# ,  - .
		class	Dll_info
			#   DLL  .
			attr_reader	:dll_file_name
			# ,     DLL.
			attr_reader	:dll_path
			# ,     (  ).
			#   nil,     .
			attr_reader	:link_name
			#  ,       .
			#   nil,     .
			attr_reader	:link_path

			def initialize(
					a_dll_file_name,
					a_dll_path,
					a_link_name,
					a_link_path )

				@dll_file_name = a_dll_file_name
				@dll_path = a_dll_path
				@link_name = a_link_name
				@link_path = a_link_path
			end
		end # class Dll_info


		# ,     .
		class	Linker_lists
			#  .
			attr_reader	:objs
			#  .
			attr_reader	:resources
			#  .
			attr_reader	:libs
			#    .
			attr_reader	:lib_paths
			#  .
			attr_reader	:linker_options

			def initialize
				@objs = Array.new
				@resources = Array.new
				@libs = Array.new
				@lib_paths = Array.new
				@linker_options = Array.new
			end

			def	add_objs( a_objs )
				push_to( @objs, a_objs )
			end

			def	add_resources( a_resources )
				push_to( @resources, a_resources )
			end

			def	add_libs( a_libs )
				push_to( @libs, a_libs )
			end

			def	add_lib_paths( a_lib_paths )
				push_to( @lib_paths, a_lib_paths )
			end

			def	add_linker_options( a_linker_options )
				push_to( @linker_options, a_linker_options )
			end

		protected
			def push_to( to, what )
				to << what
				to.flatten!
				to.uniq!
			end
		end # class Linker_lists

		# ,    DLL.
		#  -   DLL   mswin,   
		#      DLL.
		#   -   DLL   Unix,   
		#   ,       DLL.
		#       Toolset-.
		# ,   DLL   Toolset 
		#        
		# .      
		#   Dll_requirements.
		class	Dll_requirements
			#   .
			attr_reader	:libs
			#    .
			attr_reader	:lib_paths

			def	initialize
				@libs = Array.new
				@lib_paths = Array.new
			end

			def	add_libs( a_libs )
				push_to( @libs, a_libs )
			end

			def	add_lib_paths( a_lib_paths )
				push_to( @lib_paths, a_lib_paths )
			end

		protected
			def push_to( to, what )
				to << what
				to.flatten!
			end # Dll_requirements
		end

		#     'toolset'.
		#  'toolset'     
		#   .
		class	Toolset

			# ,      
			#   toolset-.
			class	Unknown_tag_ex < Mxx_ru::Ex
				def	initialize( a_tag_name )
					super( "Unknown or missing toolset tag: '#{a_tag_name}" )
				end
			end

		private
			#  toolset-.
			@name = nil
			#  .      String.
			@tags = nil

		public
			# .
			#
			# _a_name_  toolset.
			def initialize( a_name )
				@name = a_name
				@tags = Hash.new
			end

			#   .
			#     .
			#
			# _a_name_  .
			#
			# _a_value_ .   .
			def	setup_tag( a_name, a_value )
				@tags[ a_name ] = a_value
			end

			#   .
			#       ,  
			#   a_default.
			#  a_default == nil,      
			# .
			def	tag( a_name, a_default = nil )
				if !@tags.key?( a_name )
					if nil == a_default
						raise Unknown_tag_ex.new( a_name )
					end

					return a_default
				end
				return @tags[ a_name ]
			end

			#       , ,
			#   ..    runtime_mode, rtl_mode,
			# rtti_mode, threading_mode  .
			#  target    , 
			#  Mxx_ru::Cpp::Target.
			#
			#      Abstract_method_ex.
			#
			def	setup_mandatory_options( target )
				raise Abstract_method_ex.new( "Mxx_ru::Cpp::Toolset::setup_mandatory_options" )
			end

			#   C++    .
			#  target    , 
			#  Mxx_ru::Cpp::Target.
			def	make_objs( target )
				#     .
				c_objs = make_obj_names( target.mxx_c_files, target )
				cpp_objs = make_obj_names( target.mxx_cpp_files, target )

				#   ,  
				#   .
				c_objs_to_build = detect_objs_to_rebuild( c_objs )
				cpp_objs_to_build = detect_objs_to_rebuild( cpp_objs )

				#  ,   .
				if c_objs_to_build.size
					build_objs( c_objs_to_build, target,
						target.mxx_all_c_compiler_options,
						:make_c_obj_command_lines )
				end
				if cpp_objs_to_build.size
					build_objs( cpp_objs_to_build, target,
						target.mxx_all_cpp_compiler_options,
						:make_cpp_obj_command_lines )
				end

				#      
				#   target, ..     .
				c_objs.each { |o| target.obj_file( o.name ) }
				cpp_objs.each { |o| target.obj_file( o.name ) }

				#   ,    - 
				#  .
				state = Target_state::EXISTS
				state = Target_state::REBUILT if ( c_objs_to_build.size() ||
					cpp_objs_to_build.size() )

				return Target_state.new( state )
			end

			#   .
			def	clean_objs( target )
				#     .
				c_objs = make_obj_names( target.mxx_c_files, target )
				cpp_objs = make_obj_names( target.mxx_cpp_files, target )

				c_objs.each { |o| Mxx_ru::Util::delete_file( o.name ) }
				cpp_objs.each { |o| Mxx_ru::Util::delete_file( o.name ) }
			end

			#    .
			#  target    , 
			#  Mxx_ru::Cpp::Target.
			def	make_mswin_res( target )
				#   RES-    target.
				res_info = make_mswin_res_name(
					target.mxx_mswin_rc_file.name, target )

				full_name = File.join( [ res_info.path, res_info.file_name ] )
				target.mswin_res_file( full_name )

				#   .
				state = Target_state::detect( full_name,
					[ target.mxx_mswin_rc_file.name ] +
					target.mxx_mswin_rc_file.depends )
				if state.state != Target_state::EXISTS
					cmd_lines = make_mswin_res_command_lines(
						full_name, target.mxx_mswin_rc_file.name,
						target.mxx_all_mswin_rc_options, target )

					Mxx_ru::Abstract_target::run( cmd_lines, [ full_name ] )

					state = Mxx_ru::Target_state.new( Mxx_ru::Target_state::REBUILT )
				end

				return state
			end

			#       MS Windows.
			def	clean_mswin_res( target )
				#   RES-.
				res_info = make_mswin_res_name(
					target.mxx_mswin_rc_file.name, target )

				full_name = File.join( [ res_info.path, res_info.file_name ] )

				Mxx_ru::Util::delete_file( full_name )

				clean_mswin_res_specific_files( full_name, res_info, target )
			end

			#     mswin-res-.
			#      .
			#
			# _a_res_file_	  mswin-res-.
			#
			# _a_res_info_	  res- .
			#
			# _a_target_	,    mswin-res-.
			def	clean_mswin_res_specific_files(
				a_res_file,
				a_res_info,
				a_target )
			end

			#    .
			#  target    , 
			#  Mxx_ru::Cpp::Target.
			def	make_lib( target )
				#    .   
				#      lib.   
				#         
				#  .
				lib_info = make_lib_name( target.mxx_target_name, target )

				#   .
				lib_file = File.join( [ lib_info.path, lib_info.file_name ] )
				lib_state = Mxx_ru::Target_state::detect(
					lib_file, target.mxx_obj_files )
				if Mxx_ru::Target_state::EXISTS != lib_state.state
					#    .
					cmd_lines = make_lib_command_lines(
						lib_file, target.mxx_obj_files,
						target.mxx_all_librarian_options, target )

					Mxx_ru::Abstract_target::run( cmd_lines, [ lib_file ] )

					lib_state = Mxx_ru::Target_state.new( Mxx_ru::Target_state::REBUILT )
				end

				#         ,
				#    .
				target.mxx_add_full_target_name( lib_file )

				#        
				#  .
				target.mxx_add_required_lib( lib_info.link_name )
				target.mxx_add_required_lib_path( lib_info.path )

				#          .
				target.mxx_required_prjs.each { |d|
					if d.kind_of?( Mxx_ru::Binary_target )
						d.mxx_required_libs.each { |l|
							target.mxx_add_required_lib( l )
						}
						d.mxx_required_lib_paths.each { |p|
							target.mxx_add_required_lib_path( p )
						}
					end
				}

				return lib_state
			end

			#    .
			#  target    , 
			#  Mxx_ru::Cpp::Target.
			def	clean_lib( target )
				#    .   
				#      lib.
				lib_info = make_lib_name( target.mxx_target_name, target )

				lib_file = File.join( [ lib_info.path, lib_info.file_name ] )
				Mxx_ru::Util::delete_file( lib_file )

				clean_lib_specific_files( lib_file, lib_info, target )
			end

			#    ,
			#       .
			#      .
			#      .
			#
			# _a_lib_file_	  -.
			#
			# _a_lib_info_	   Lib_info.
			#
			# _a_target_	,    .
			def	clean_lib_specific_files(
				a_lib_file,
				a_lib_info,
				a_target )
			end

			#   - .
			#  target    , 
			#  Mxx_ru::Cpp::Target.
			def	make_dll( target )
				#   ,     .
				dll_info = make_dll_name( target.mxx_target_name, target )

				#    -.
				all_depend_files = create_executable_depend_list( target )

				#      
				#    DLL.
				link_lists = prepare_linker_lists( target )

				dll_file = File.join( [ dll_info.dll_path,  dll_info.dll_file_name ] )
				dll_state = Mxx_ru::Target_state::detect(
					dll_file, all_depend_files )
				if Mxx_ru::Target_state::EXISTS != dll_state.state
					#    .

					cmd_lines = make_dll_command_lines(
						dll_file, dll_info, link_lists, target )

					Mxx_ru::Abstract_target::run( cmd_lines, [ dll_file ] )

					dll_state = Mxx_ru::Target_state.new( Mxx_ru::Target_state::REBUILT )
				end

				#         ,
				#    .
				target.mxx_add_full_target_name( dll_file )

				# ,     ,  
				#    DLL.
				dll_requirements = make_dll_requirements(
					dll_file, dll_info, link_lists, target )

				dll_requirements.libs.each { |l|
						target.mxx_add_required_lib( l ) }
				dll_requirements.lib_paths.each { |p|
						target.mxx_add_required_lib_path( p ) }

				return dll_state
			end

			#   - .
			#  target    , 
			#  Mxx_ru::Cpp::Target.
			def	clean_dll( target )
				#   ,     .
				dll_info = make_dll_name( target.mxx_target_name, target )

				dll_file = File.join( [ dll_info.dll_path,  dll_info.dll_file_name ] )
				Mxx_ru::Util::delete_file( dll_file )

				clean_dll_specific_files( dll_file, dll_info, target )
			end

			#    ,
			#       .
			#      ,  
			#    DLL    
			# .
			#
			#      .
			#
			# _a_dll_file_	  -.
			#
			# _a_dll_info_	   Dll_info.
			#
			# _a_target_	,    .
			def	clean_dll_specific_files(
				a_dll_file,
				a_dll_info,
				a_target )
			end

			#     (exe).
			#  target    , 
			#  Mxx_ru::Cpp::Target.
			def	make_exe( target )
				#   ,     .
				exe_info = make_exe_name( target.mxx_target_name, target )

				#    -.
				all_depend_files = create_executable_depend_list( target )

				exe_file = File.join( [ exe_info.path,  exe_info.file_name ] )
				exe_state = Mxx_ru::Target_state::detect(
					exe_file, all_depend_files )
				if Mxx_ru::Target_state::EXISTS != exe_state.state
					#    .

					link_lists = prepare_linker_lists( target )
					cmd_lines = make_exe_command_lines(
						exe_file, exe_info, link_lists, target )

					Mxx_ru::Abstract_target::run( cmd_lines, [ exe_file ] )

					exe_state = Mxx_ru::Target_state.new( Mxx_ru::Target_state::REBUILT )
				end

				#         ,
				#    .
				target.mxx_add_full_target_name( exe_file )

				return exe_state
			end

			#     (exe).
			#  target    , 
			#  Mxx_ru::Cpp::Target.
			def	clean_exe( target )
				#   ,     .
				exe_info = make_exe_name( target.mxx_target_name, target )

				exe_file = File.join( [ exe_info.path,  exe_info.file_name ] )
				Mxx_ru::Util::delete_file( exe_file )

				clean_exe_specific_files( exe_file, exe_info, target )
			end

			#    ,
			#   EXE-   .
			#      .
			#
			#      .
			#
			# _a_exe_file_	  .
			#
			# _a_exe_info_	   Exe_info.
			#
			# _a_target_	,    .
			def	clean_exe_specific_files(
				a_exe_file,
				a_exe_info,
				a_target )
			end

			#     .
			#      .
			#
			#      Abstract_method_ex.
			#
			def	obj_file_ext()
				raise Abstract_method_ex.new( "Mxx_ru::Cpp::Toolset::obj_file_ext" )
			end

			#       
			#  C .
			#  Array  String   .
			#      .
			#
			#      Abstract_method_ex.
			#
			# _obj_name_	  .
			#
			# _source_name_	  .
			#
			# _compiler_options_	  .
			# Array  String.
			#
			# _target_ ,      .
			def	make_c_obj_command_lines(
				obj_name,
				source_name,
				compiler_options,
				target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::make_c_obj_command_lines" )
			end

			#       
			#  C++ .
			#  Array  String   .
			#      .
			#
			#      Abstract_method_ex.
			#
			# _obj_name_	  .
			#
			# _source_name_	  .
			#
			# _compiler_options_	  .
			# Array  String.
			#
			# _target_ ,      .
			def	make_cpp_obj_command_lines(
				obj_name,
				source_name,
				compiler_options,
				target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::make_c_obj_command_lines" )
			end

			#      mswin-.
			#  Array  String   .
			#      .
			#
			#      Abstract_method_ex.
			#
			# _res_name_	   res-.
			#
			# _rc_file_	  .
			#
			# _rc_options_	  .
			#
			# _target_	,    .
			def	make_mswin_res_command_lines(
				res_name,
				rc_file,
				rc_options,
				target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::make_mswin_res_command_lines" )
			end

			#     .
			#
			#      Abstract_method_ex.
			#
			def	lib_file_name( source_name )
				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::lib_file_name" )
			end

			#    ,  
			#   .
			#
			#      Abstract_method_ex.
			#
			def	lib_link_name( source_name )
				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::lib_link_name" )
			end

			#       .
			#  Array  String   .
			#      .
			#
			#      Abstract_method_ex.
			#
			# _lib_name_	  .
			#
			# _obj_files_	  ,     .
			#
			# _librarian_options_	  .
			# Array  String.
			#
			# _target_	,      .
			def	make_lib_command_lines(
				lib_name,
				obj_files,
				librarian_options,
				target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::make_lib_command_lines" )
			end

			#    - .
			#
			#      Abstract_method_ex.
			#
			def	dll_file_name( source_name )
				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::dll_file_name" )
			end

			#    ,  
			#   .
			#   nil,    .
			#      .
			#
			#      Abstract_method_ex.
			#
			# _dll_real_name_	,     dll_file_name.
			#
			# _target_	,     .
			def	implib_link_name(
				dll_real_name,
				target )
				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::implib_link_name" )
			end

			#      ,  
			#   .
			# , ,   implib_link_name
			#    nil .
			#   nil,    .
			#      .
			#
			#      Abstract_method_ex.
			#
			# _dll_real_name_	,     dll_file_name.
			#
			# _dll_real_path_	 ,     DLL.
			#
			# _target_	,     .
			def	implib_link_path(
				dll_real_name,
				dll_real_path,
				target )
				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::implib_link_path" )
			end

			#      DLL.
			#  Array  String   .
			#      .
			#
			#      Abstract_method_ex.
			#
			# _a_dll_name_	  DLL.
			#
			# _a_dll_info_	 DLL.
			#
			# _a_linker_lists_	  .
			#
			# _a_target_	,    DLL.
			def	make_dll_command_lines(
				a_dll_name,
				a_dll_info,
				a_linker_lists,
				a_target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::make_dll_command_lines" )
			end

			#      DLL.
			#   Dll_requirements.
			#
			#      Abstract_method_ex.
			#
			# _a_dll_name_	  DLL.
			#
			# _a_dll_info_	 DLL.
			#
			# _a_linker_lists_	  .
			#
			# _a_target_	,    DLL.
			def make_dll_requirements(
				a_dll_name,
				a_dll_info,
				a_linker_lists,
				a_target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::make_dll_requirements" )
			end

			#   EXE-.
			#
			#      Abstract_method_ex.
			#
			def	exe_file_name( source_name )
				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::exe_file_name" )
			end

			#      EXE.
			#  Array  String   .
			#      .
			#
			#      Abstract_method_ex.
			#
			# _a_exe_name_	  EXE.
			#
			# _a_exe_info_	 EXE.
			#
			# _a_linker_lists_	  .
			#
			# _a_target_	,    EXE.
			def	make_exe_command_lines(
				a_exe_name,
				a_exe_info,
				a_linker_lists,
				a_target )

				raise Abstract_method_ex.new(
					"Mxx_ru::Cpp::Toolset::make_exe_command_lines" )
			end

			#   toolset-.
			def	name
				return @name
			end

		protected
			#    .
			#    Obj_info.
			def	make_obj_names( sources, target )
				obj_names = Array.new
				sources.each { |s|
						#    .
						only_name = File::basename( s.name )
						#    .
						only_name = Mxx_ru::Util::remove_file_ext( only_name )

						#    .
						source_path = File::dirname( s.name )
						#   ,    
						#   .
						dest_path = target.mxx_obj_placement.get_obj(
							source_path, self, target )
						#    .
						obj_names.push( Obj_info.new(
							File::join( dest_path, only_name + obj_file_ext() ),
							s,
							s.compiler_options ) )
					}
				return obj_names
			end

			#   ,   
			#  (   )  
			# (   ).
			#    Obj_info.
			#
			# _objs_	Array  Obj_info
			def	detect_objs_to_rebuild( objs )

				obj_names = Array.new
				objs.each { |o|
						obj_state = Mxx_ru::Target_state::detect(
							o.name, [ o.source.name ] + o.source.depends )
						if Mxx_ru::Target_state::EXISTS != obj_state.state
							#    .
							obj_names << o
						end
					}

				return obj_names
			end

			#    .
			#
			# _objs_to_build_	Array  Obj_info.
			#
			# _target_	-,      .
			#
			# _lang_specific_options_	  C  C++  .
			# Array  String.
			#
			# _a_method_name_	 ,    
			#      .
			def	build_objs(
				objs_to_build,
				target,
				lang_specific_options,
				a_method_name )

				all_files_options = target.mxx_all_compiler_options +
					lang_specific_options;
				all_files_options.flatten!
				all_files_options.uniq!

				objs_to_build.each { |o|
						#      .
						compiler_options =  all_files_options + o.options;
						compiler_options.flatten!
						compiler_options.uniq!

						cmd_lines = self.send( a_method_name,
							o.name, o.source.name,
							compiler_options,
							target )

						#  .
						Mxx_ru::Abstract_target::run( cmd_lines, [ o.name ] )
					}

			end

			#     mswin .
			#   Mswin_res_info.
			def	make_mswin_res_name( rc_file, target )
				source_name = Mxx_ru::Util::remove_file_ext(
					File::basename( rc_file ) )
				source_path = File::dirname( rc_file )

				real_name = mswin_res_file_name( source_name )
				dest_path = target.mxx_obj_placement.get_mswin_res(
					source_path, self, target )

				return Mswin_res_info.new( real_name, dest_path )
			end

			#    .
			#   Lib_info.
			def	make_lib_name( lib_name, target )
				source_name = File::basename( lib_name )
				source_path = File::dirname( lib_name )

				real_name = lib_file_name( source_name )
				link_name = lib_link_name( source_name )
				dest_path = target.mxx_obj_placement.get_lib(
					source_path, self, target )

				return Lib_info.new(
					real_name, dest_path, link_name )
			end

			#   - .
			#   Dll_info.
			def	make_dll_name( dll_name, target )
				source_name = File::basename( dll_name )
				source_path = File::dirname( dll_name )

				dll_real_name = dll_file_name( source_name )
				dll_dest_path = target.mxx_obj_placement.get_dll(
					source_path, self, target )

				link_name = implib_link_name( dll_real_name, target )
				link_path = nil
				if nil != link_name
					link_path = implib_link_path(
						dll_real_name, dll_dest_path, target )
				end

				return Dll_info.new(
					dll_real_name, dll_dest_path, link_name, link_path )
			end

			#   EXE-.
			#   Dll_info.
			def	make_exe_name( exe_name, target )
				source_name = File::basename( exe_name )
				source_path = File::dirname( exe_name )

				real_name = exe_file_name( source_name )
				dest_path = target.mxx_obj_placement.get_exe(
					source_path, self, target )

				return Exe_info.new(
					real_name, dest_path )
			end

			#    ,   
			#  EXE  DLL .
			def	create_executable_depend_list( target )
				all_depend_files = Array.new
				target.mxx_required_prjs.each { |d|
						depends = d.mxx_full_targets_names
						if nil != depends
							all_depend_files << depends
						end
					}
				all_depend_files << target.mxx_obj_files

				if target.mxx_mswin_res_file
					all_depend_files << target.mxx_mswin_res_file
				end

				all_depend_files.flatten!

				return all_depend_files
			end

			#     .
			#    Linker_lists
			def	prepare_linker_lists( target )
				result = Linker_lists.new

				result.add_objs( target.mxx_obj_files )
				result.add_resources( target.mxx_mswin_res_file )
				result.add_libs( target.mxx_required_libs )
				result.add_lib_paths( target.mxx_required_lib_paths )

				target.mxx_required_prjs.each { |d|
					if d.kind_of?( Mxx_ru::Binary_target )
						result.add_libs( d.mxx_required_libs )
						result.add_lib_paths( d.mxx_required_lib_paths )
					end
				}

				result.add_linker_options( target.mxx_all_linker_options )

				return result
			end

		end # class Toolset

		#  toolset.
		@@current_toolset = nil

		#   toolset.
		def Cpp.setup_toolset( toolset )
			raise Mxx_ru.Invalid_cast_ex.new(
				'Expected Mxx_ru::Cpp::Toolset' ) if
					!( toolset.kind_of?( Mxx_ru::Cpp::Toolset ) )

			@@current_toolset = toolset
		end

		#   toolset.
		def Cpp.toolset
			return @@current_toolset
		end

	end # module Cpp

end # module Mxx_ru
