#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require 'mxx_ru/cpp/toolset'

module	Mxx_ru
	module	Cpp
		module Toolsets

			#  toolset  Borland C++   Win32.
			class	Bcc_win32_family < Mxx_ru::Cpp::Toolset
			public
				def initialize( a_name = "bcc" )
					super( a_name )

					setup_tag( "host_os", "mswin" )
					setup_tag( "target_os", "mswin" )
				end

				#   .
				def	compiler_name
					return "bcc32"
				end

				#   .
				def	linker_name
					return "ilink32"
				end

				#   .
				def	librarian_name
					return "tlib"
				end

				#    .
				def	rc_name
					return "brc32"
				end

				#     .
				def	import_librarian_name
					return "implib"
				end

				#   . Mxx_ru::Cpp::Toolset#setup_mandatory_options.
				def	setup_mandatory_options( target )

					# Disable warning: Parameter is never used.
					target.compiler_option( "-w-par" )
					# Disable warning: <ID> is assigned a value that is never used.
					target.compiler_option( "-w-aus" )

					# Turn all warnings on.
					target.linker_option( "/w" )
					# Don't generate state files (disable incremental linking).
					target.linker_option( "/Gn" )
					# Suppress creation of map file.
					target.linker_option( "/x" )
					# Suppress command line banner.
					target.linker_option( "/q" )

					if RUNTIME_DEBUG == target.mxx_runtime_mode
						target.compiler_option( "-v" )
						target.compiler_option( "-y" )
						target.linker_option( "/v" )
					elsif RUNTIME_RELEASE == target.mxx_runtime_mode
						target.define( "NDEBUG" )
						if OPTIM_SIZE == target.mxx_optimization
							target.compiler_option( "-O1" )
						else
							target.compiler_option( "-O2" )
						end
					end

					if RTTI_ENABLED == target.mxx_rtti_mode
						target.cpp_compiler_option( "-RT" )
					elsif RTTI_DISABLED == target.mxx_rtti_mode
						target.cpp_compiler_option( "-RT-" )
					end

					if RTL_SHARED == target.mxx_rtl_mode
						target.compiler_option( "-WR" )
					end

					if THREADING_MULTI == target.mxx_threading_mode
						target.compiler_option( "-tWM" )
						if RTL_SHARED == target.mxx_rtl_mode
							target.lib( "cw32mti.lib" )
						else
							target.lib( "cw32mt.lib" )
						end
					else
						if RTL_SHARED == target.mxx_rtl_mode
							target.lib( "cw32i.lib" )
						else
							target.lib( "cw32.lib" )
						end
					end

					if target.target_type.name == Dll_target_type::TYPE
						target.compiler_option( "-WD" )
						target.linker_option( "/Tpd" )
					elsif target.target_type.name == Exe_target_type::TYPE
						if SCREEN_WINDOW == target.mxx_screen_mode
							target.linker_option( "/aa" )
						else
							target.linker_option( "/ap" )
						end

						target.linker_option( "/Tpe" )
					end

					target.lib( "import32.lib" )

					if SCREEN_WINDOW == target.mxx_screen_mode
						target.compiler_option( "-tW" )
					else
						target.compiler_option( "-tWC" )
					end

					#  defines   include_path    
					#  .
					target.mxx_all_defines.each { |d|
						target.compiler_option( "-D" + d )
						target.mswin_rc_option( "-d" + d )
					}

					target.mxx_all_include_paths.each { |p|
						target.compiler_option( "-I" + p )
						target.mswin_rc_option( "-i" + p )
					}

					#         .
					target.mxx_all_mswin_rc_defines.each { |d|
						target.mswin_rc_option( "-d" + d )
					}
					target.mxx_all_mswin_rc_include_paths.each { |p|
						target.mswin_rc_option( "-i" + p )
					}

				end

				#   . Mxx_ru::Cpp::Toolset#clean_dll_specific_files.
				#
				#    ,   .
				def	clean_dll_specific_files(
					a_dll_file,
					a_dll_info,
					a_target )

					#    ,    .
					if nil != a_dll_info.link_name
						implib_name = File.join(
							[ a_dll_info.link_path, a_dll_info.link_name ] )
						Mxx_ru::Util::delete_file( implib_name )
					end

					#     TDS .
					Mxx_ru::Util::delete_file( tds_file_name( a_dll_file ) )
				end

				#   . Mxx_ru::Cpp::Toolset#clean_exe_specific_files.
				def	clean_exe_specific_files(
					a_exe_file,
					a_exe_info,
					a_target )

					Mxx_ru::Util::delete_file( tds_file_name( a_exe_file ) )
				end

				#   . Mxx_ru::Cpp::Toolset#obj_file_ext.
				def	obj_file_ext
					return ".obj"
				end

				#   . Mxx_ru::Cpp::Toolset#make_c_obj_command_lines.
				def	make_c_obj_command_lines(
					obj_name,
					source_name,
					compiler_options,
					target )

					tmpfile = Mxx_ru::Util::Tmp_files.instance.create(
						"-c -P- -o#{obj_name} " +
						"#{compiler_options.join(' ')} #{source_name}" )

					return [ "#{compiler_name} @#{tmpfile}" ]
				end

				#   . Mxx_ru::Cpp::Toolset#make_c_obj_command_lines.
				def	make_cpp_obj_command_lines(
					obj_name,
					source_name,
					compiler_options,
					target )

					tmpfile = Mxx_ru::Util::Tmp_files.instance.create(
						"-c -P -o#{obj_name} " +
						"#{compiler_options.join(' ')} #{source_name}" )

					return [ "#{compiler_name} @#{tmpfile}" ]
				end

				#   . Mxx_ru::Cpp::Toolset#mswin_res_file_name.
				def	mswin_res_file_name( source_name )
					return source_name + ".res"
				end

				#   . Mxx_ru::Cpp::Toolset#make_mswin_res_command_lines.
				def	make_mswin_res_command_lines(
					res_name,
					rc_file,
					rc_options,
					target )

					return [ "#{rc_name} " +
						"#{rc_options.join(' ')} -r " +
						"-fo#{res_name} #{rc_file}" ]
				end

				#   . Mxx_ru::Cpp::Toolset#lib_file_name.
				def	lib_file_name( source_name )
					return source_name + ".lib"
				end

				#   . Mxx_ru::Cpp::Toolset#lib_link_name.
				def	lib_link_name( source_name )
					return lib_file_name( source_name )
				end

				#   . Mxx_ru::Cpp::Toolset#make_lib_command_lines.
				def	make_lib_command_lines(
					lib_name,
					obj_files,
					librarian_options,
					target )

					obj_files_commands = String.new
					obj_files.each { |o|
						obj_files_commands += "-+\"#{unix2win(o)}\" "
					}

					tmpfile = Mxx_ru::Util::Tmp_files.instance.create(
						"#{librarian_options.join(' ')} " +
						"\"#{unix2win(lib_name)}\" " +
						"#{obj_files_commands}" )

					return [ "#{librarian_name} @#{tmpfile}" ]
				end

				#   . Mxx_ru::Cpp::Toolset#dll_file_name.
				def	dll_file_name( source_name )
					return source_name + ".dll"
				end

				#   . Mxx_ru::Cpp::Toolset#implib_link_name.
				def	implib_link_name(
					dll_real_name,
					target )

					#  Visual C++      .
					if nil != target.mxx_implib_path
						return lib_file_name( target.mxx_target_name )
					end

					return nil
				end

				#   . Mxx_ru::Cpp::Toolset#implib_link_path.
				#
				#  ,   
				# Mxx_ru::Cpp::Target#mxx_implib_path.
				def	implib_link_path(
					dll_real_name,
					dll_real_path,
					target )
					return target.mxx_obj_placement.get_lib(
						target.mxx_implib_path, self, target )
				end

				#   . Mxx_ru::Cpp::Toolset#make_dll_command_lines.
				def	make_dll_command_lines(
					a_dll_name,
					a_dll_info,
					a_linker_lists,
					a_target )

					result = Array.new

					response_file = generate_linker_respfile(
						a_dll_name, "c0d32.obj", a_linker_lists )

					result << "#{linker_name} @#{response_file}"

					if nil != a_dll_info.link_name
						implib_name = File.join( [ a_dll_info.link_path,
							a_dll_info.link_name ] )
						result << "#{import_librarian_name} #{implib_name} " +
							"#{a_dll_name}"
					end

					if RUNTIME_DEBUG != a_target.mxx_runtime_mode
						result << "if exist #{a_dll_name} del " +
							"#{tds_file_name(a_dll_name)}"
					end

					return result
				end

				#   . Mxx_ru::Cpp::Toolset#make_dll_requirements.
				def make_dll_requirements(
					a_dll_name,
					a_dll_info,
					a_linker_lists,
					a_target )

					result = Dll_requirements.new

					#  ,     .
					if nil != a_dll_info.link_name
						result.add_libs( [ a_dll_info.link_name ] )
						result.add_lib_paths( [ a_dll_info.link_path ] )
					end

					return result
				end

				#   . Mxx_ru::Cpp::Toolset#exe_file_name.
				def	exe_file_name( source_name )
					return source_name + ".exe"
				end

				#   . Mxx_ru::Cpp::Toolset#make_exe_command_lines.
				def	make_exe_command_lines(
					a_exe_name,
					a_exe_info,
					a_linker_lists,
					a_target )

					result = Array.new

					if SCREEN_WINDOW == a_target.mxx_screen_mode
						startup_obj = "c0w32.obj"
					else
						startup_obj = "c0x32.obj"
					end

					response_file = generate_linker_respfile(
						a_exe_name, startup_obj, a_linker_lists )

					result << "#{linker_name} @#{response_file}"

					if RUNTIME_DEBUG != a_target.mxx_runtime_mode
						result << "if exist #{unix2win(a_exe_name)} del " +
							"#{tds_file_name(a_exe_name)}"
					end

					return result
				end

			protected
				#   TDS    DLL  EXE.
				def	tds_file_name( a_source_name )
					return unix2win(
						Mxx_ru::Util::remove_file_ext( a_source_name ) +
						".tds" )
				end

				#   .
				#
				#  unix-   
				# windows-.
				#
				#        .
				def	unix2win( a_name )
					return "\"" + a_name.gsub( "/", "\\" ) + "\""
				end

				#    .
				#
				#  unix-   
				# windows-.
				#
				#      .
				def	unix2win_mass( a_names )
					return a_names.collect { |d|
						if d
							unix2win( d )
						end
					}
				end

				#  response-   .
				#
				# _a_output_name_   .
				#
				# _a_startup_objfile_     startup-.
				#
				# _a_linker_list_    , ,
				#     ..
				#
				#   response-.
				def	generate_linker_respfile(
					a_output_name,
					a_startup_objfile,
					a_linker_lists )

					result = "#{a_linker_lists.linker_options.join(' ')} "

					if 0 != a_linker_lists.lib_paths.size
						result << "/L;"
						a_linker_lists.lib_paths.each { |p| result << "#{p};" }
						result << " "
					end

					result << "#{a_startup_objfile} "

					result << "#{unix2win_mass(a_linker_lists.objs).join(' ')} "
					result << ", #{unix2win(a_output_name)}, , "
					result << "#{unix2win_mass(a_linker_lists.libs).join(' ')}, , "
					result << "#{unix2win_mass(a_linker_lists.resources).join(' ')} "

					return Mxx_ru::Util::Tmp_files.instance.create( result )
				end

			end # class Bcc_win32_family

		end # module Toolsets

	end # module Cpp

end # module Mxx_ru
