#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require 'mxx_ru/cpp/toolset'

module	Mxx_ru
	module	Cpp
		module Toolsets

			#  toolset  GCC (C/C++).
			class	Gcc_family < Mxx_ru::Cpp::Toolset
			public
				#  ,  ,    gcc .
				GCC_PORT_TAG	= "gcc_port"
				# GCC_PORT: GCC  Unix/Linux.
				GCC_PORT_UNIX = "unix"
				# GCC_PORT: MinGW  Win32.
				GCC_PORT_MINGW = "mingw"
				# GCC_PORT: Cygwin  Win32.
				GCC_PORT_CYGWIN = "cygwin"

				def initialize( a_name )
					super( a_name )
				end

				#    .
				def	c_compiler_name
					return "gcc"
				end

				#   ++ .
				def	cpp_compiler_name
					return "g++"
				end

				#   .
				def	linker_name
					return "g++"
				end

				#   .
				def	librarian_name
					return "ar"
				end

				#   . Mxx_ru::Cpp::Toolset#setup_mandatory_options.
				def	setup_mandatory_options( target )

					if RUNTIME_DEBUG == target.mxx_runtime_mode
						target.compiler_option( "-g" )
						target.linker_option( "-g" )
					elsif RUNTIME_RELEASE == target.mxx_runtime_mode
						target.define( "NDEBUG" )
						target.linker_option( "-s" )
						if OPTIM_SIZE == target.mxx_optimization
							target.compiler_option( "-Os" )
						else
							target.compiler_option( "-O2" )
						end
					end

					if RTTI_DISABLED == target.mxx_rtti_mode
						target.cpp_compiler_option( "-fno-rtti" )
					end

					if RTL_STATIC == target.mxx_rtl_mode
						target.linker_option( "--static-libgcc" )
					else
						target.linker_option( "--shared-libgcc" )
					end

#   mswin     .
					if target.target_type.name == Dll_target_type::TYPE
						target.compiler_option( "-fPIC" )
					end

					#   C++ ,    
					#  stdc++.
					if target.mxx_cpp_files.size
#						target.lib( "stdc++" )
					end

					target.mxx_all_defines.each { |d|
						target.compiler_option( "-D" + d )
					}

					target.mxx_all_include_paths.each { |p|
						target.compiler_option( "-I" + p )
					}
				end

				#   . Mxx_ru::Cpp::Toolset#obj_file_ext.
				def	obj_file_ext
					return ".o"
				end

				#   . Mxx_ru::Cpp::Toolset#make_c_obj_command_lines.
				def	make_c_obj_command_lines(
					obj_name,
					source_name,
					compiler_options,
					target )

					cmd_line = "-c -o #{obj_name} " +
						"#{compiler_options.join(' ')} #{source_name}"

					return [ "#{c_compiler_name} #{cmd_line}" ]
				end

				#   . Mxx_ru::Cpp::Toolset#make_c_obj_command_lines.
				def	make_cpp_obj_command_lines(
					obj_name,
					source_name,
					compiler_options,
					target )

					cmd_line = "-c -o #{obj_name} " +
						"#{compiler_options.join(' ')} #{source_name}"

					return [ "#{cpp_compiler_name} #{cmd_line}" ]
				end

				#   . Mxx_ru::Cpp::Toolset#lib_file_name.
				def	lib_file_name( source_name )
					return "lib" + source_name + ".a"
				end

				#   . Mxx_ru::Cpp::Toolset#lib_link_name.
				def	lib_link_name( source_name )
					return source_name
				end

				#   . Mxx_ru::Cpp::Toolset#make_lib_command_lines.
				def	make_lib_command_lines(
					lib_name,
					obj_files,
					librarian_options,
					target )

					result = "-r #{librarian_options.join(' ')} " +
						"#{lib_name} #{obj_files.join(' ')}"

					return [ "#{librarian_name} #{result}" ]
				end

				#   . Mxx_ru::Cpp::Toolset#dll_file_name.
				def	dll_file_name( source_name )
					return "lib" + source_name + ".so"
				end

				#   . Mxx_ru::Cpp::Toolset#implib_link_name.
				def	implib_link_name(
					dll_real_name,
					target )

					#   UNIX    
					#   .
					return lib_link_name(
						File.basename( target.mxx_target_name ) )
				end

				#   . Mxx_ru::Cpp::Toolset#implib_link_path.
				def	implib_link_path(
					dll_real_name,
					dll_real_path,
					target )

					#   UNIX DLL   
					# ,    dll_real_path.
					return dll_real_path
				end

				#   . Mxx_ru::Cpp::Toolset#make_dll_command_lines.
				def	make_dll_command_lines(
					a_dll_name,
					a_dll_info,
					a_linker_lists,
					a_target )

					result = "-shared " +
						"#{a_linker_lists.linker_options.join(' ')} " +
						"-o #{a_dll_name} "

					a_linker_lists.lib_paths.each { |p| result << "-L#{p} " }

					result << "#{a_linker_lists.objs.join(' ')} "
					a_linker_lists.libs.each { |l| result << "-l#{l} " }

					result << port_specific_dll_link_options(
						a_dll_name, a_dll_info, a_linker_lists, a_target )

					return [ "#{linker_name} #{result}" ]
				end

				#   . Mxx_ru::Cpp::Toolset#make_dll_requirements.
				def make_dll_requirements(
					a_dll_name,
					a_dll_info,
					a_linker_lists,
					a_target )

					result = Dll_requirements.new

					#  UNIX      
					# ,     .

					#  ,   .
					result.add_libs( [ a_dll_info.link_name ] )
					result.add_lib_paths( [ a_dll_info.link_path ] )

					#          .
					a_target.mxx_required_prjs.each { |d|
						if d.kind_of?( Mxx_ru::Binary_target )
							result.add_libs( d.mxx_required_libs )
							result.add_lib_paths( d.mxx_required_lib_paths )
						end
					}

					return result
				end

				#   . Mxx_ru::Cpp::Toolset#exe_file_name.
				def	exe_file_name( source_name )
					return source_name
				end

				#   . Mxx_ru::Cpp::Toolset#make_exe_command_lines.
				def	make_exe_command_lines(
					a_exe_name,
					a_exe_info,
					a_linker_lists,
					a_target )

					result = "#{a_linker_lists.linker_options.join(' ')} " +
						"-o #{a_exe_name} "

					a_linker_lists.lib_paths.each { |p| result << "-L#{p} " }

					result << "#{a_linker_lists.objs.join(' ')} "
					a_linker_lists.libs.each { |l| result << "-l#{l} " }

					result << port_specific_exe_link_options(
						a_exe_name, a_exe_info, a_linker_lists, a_target )

					return [ "#{linker_name} #{result}" ]
				end

				#  ,     
				#     DLL.
				#
				#     make_dll_command_lines.
				#
				#      .
				def	port_specific_dll_link_options(
					a_dll_name, a_dll_info, a_linker_lists, a_target )
					return ""
				end

				#  ,     
				#     EXE.
				#
				#     make_exe_command_lines.
				#
				#      .
				def	port_specific_exe_link_options(
					a_exe_name, a_exe_info, a_linker_lists, a_target )
					return ""
				end

			end # class Gcc_family

			#  toolset   GCC (C/C++)   Win32.
			#        MinGW,    Cygwin.
			class	Gcc_mswin_family < Gcc_family
			public

				#   . Mxx_ru::Cpp::Toolset#exe_file_name.
				def	exe_file_name( source_name )
					return source_name + ".exe"
				end

				#   . Mxx_ru::Cpp::Toolset#clean_dll_specific_files.
				#
				#    ,   .
				def	clean_dll_specific_files(
					a_dll_file,
					a_dll_info,
					a_target )

					#    ,    .
					if nil != a_dll_info.link_name
						implib_name = File.join( [ a_dll_info.link_path,
							lib_file_name( a_dll_info.link_name ) ] )
						Mxx_ru::Util::delete_file( implib_name )
					end
				end

				#   . Mxx_ru::Cpp::Toolset#implib_link_name.
				def	implib_link_name(
					dll_real_name,
					target )

					#   mswin    
					#   .
					if nil != target.mxx_implib_path
						return lib_link_name( target.mxx_target_name )
					end

					return nil
				end

				#   . Mxx_ru::Cpp::Toolset#implib_link_path.
				def	implib_link_path(
					dll_real_name,
					dll_real_path,
					target )
					return target.mxx_obj_placement.get_lib(
						target.mxx_implib_path, self, target )
				end

				#   . Mxx_ru::Cpp::Toolset#make_dll_requirements.
				def make_dll_requirements(
					a_dll_name,
					a_dll_info,
					a_linker_lists,
					a_target )

					result = Dll_requirements.new

					#  ,     .
					if nil != a_dll_info.link_name
						result.add_libs( [ a_dll_info.link_name ] )
						result.add_lib_paths( [ a_dll_info.link_path ] )
					end

					return result
				end

				#  ,     
				#     DLL.
				#
				#     make_dll_command_lines.
				#
				#      .
				def	port_specific_dll_link_options(
					a_dll_name, a_dll_info, a_linker_lists, a_target )
					#    ,   
					# .
					if nil != a_dll_info.link_name
						full_lib_name = File.join( [ a_dll_info.link_path,
							lib_file_name( a_dll_info.link_name ) ] )

						return "-Wl,--out-implib=#{full_lib_name}" +
							",--export-all-symbols"
					end

					return ""
				end

			end # class Gcc_mswin_family

		end # module Toolsets

	end # module Cpp

end # module Mxx_ru
