#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

module	Mxx_ru

	#     .
	class	Ex < Exception
	end

	# ,     '  '.
	class	Abstract_method_ex < Ex
		def initialize( method_name )
			super( "Pure virtual method call: " + method_name )
		end
	end

	# ,  ,      ,
	#  .
	class	Invalid_cast_ex < Ex
		def initialize( msg )
			super( msg )
		end
	end

	# ,      
	#   .
	class	Build_ex < Ex
		def initialize( cmd_line, desc )
			super( "Build error: '#{cmd_line}' returns '#{desc}'" )
		end
	end

	# ,  ,   
	#  .
	class	Unsupported_target_type_ex < Ex
		def initialize( target_class, target_type )
			super( "Unsupported target type '#{target_type}'" +
				" in '#{target_class}'" )
		end
	end

	# ,  ,    
	#    target.
	class	Target_unknown_ex < Ex
		def initialize( target_name )
			super( "Target unknown for: " + target_name )
		end
	end

	# ,  ,    
	#    runtime_mode, rtl_mode, ...
	class	Global_option_conflict_ex < Ex
		def initialize(
			#  .
			a_name,
			#   .
			a_current,
			#    .
			a_current_owner,
			#   .
			a_new,
			#     .
			a_new_owner )

			super( "Global option '#{a_name}' conflict detected! " +
				"Current value '#{a_current}' set by '#{a_current_owner}'. " +
				"New value '#{a_new}' required by '#{a_new_owner}'" )
		end
	end

	# ,  , 
	#    runtime_mode, rtl_mode, ..., 
	#   .
	class	Global_option_disallowed_value_ex < Ex
		def initialize(
			#  .
			a_name,
			#   .
			#   Array  String.
			a_available_values,
			#   .
			a_new,
			#     .
			a_new_owner )

			super( "Value '#{new}' required by '#{a_new_owner}' " +
				"disallowed for global option '#{a_name}'! " +
				"Allowed values are: #{a_available_values.join(' ')}" )
		end
	end

	# ,  ,  -  
	#  toolset-.
	class	Unsupported_mode_ex < Ex
		def	initialize( msg )
			super( msg )
		end
	end

	# ,  ,   - -,
	#  ,    .
	class	Missing_depend_file_ex < Ex
		def initialize( missing_file, depend_for )
			super( "File '#{missing_file}' required by '#{depend_for}' " +
				"missing" )
		end
	end

	# ,  ,  -  
	#    .
	class	Invalid_value_ex < Ex
		def	initialize( msg )
			super( msg )
		end
	end

end
