#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004-2005, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require 'mxx_ru/abstract_target.rb'

module Mxx_ru

# ,       make-.
#
# ,       typegen 
#  ddl- cpp-  hpp-.      
#   :
#	generator( Mxx_ru::Makestyle_generator.new(
#		[ "myfile.cpp", "myfile.hpp" ],
#		[ "myfile.ddl" ],
#		[ "typegen --from myfile.ddl --cpp myfile.cpp --header myfile.hpp" ] ) )
#
# .  C++      sources_root.
class	Makestyle_generator < Abstract_generator
	#   ,      .
	attr_reader	:target_files

	#   ,     .
	attr_reader	:dependencies

	#  ,      ,
	#    .
	attr_reader	:build_cmds

	#  ,      ,
	#    .
	#
	#    ,     clean 
	#    .
	attr_reader	:clean_cmds

	# .
	#
	# _a_target_files_   .
	#
	# _a_dependencies_  .
	#
	# _a_build_cmds_	     .
	#
	# _a_clean_cmds_	     
	#    clean.
	#
	# .       
	# ,    . :
	#	Mxx_ru::Makestyle_generator.new( "myfile.cpp", "myfile.ddl",
	#		[ "typegen --from myfile.ddl --to myfile.cpp" ] )
	#  :
	#	Mxx_ru::Makestyle_generator.new( [ "myfile.cpp" ], [ "myfile.ddl" ],
	#		"typegen --from myfile.ddl --to myfile.cpp" )
	#
	def	initialize(
		a_target_files,
		a_dependencies,
		a_build_cmds,
		a_clean_cmds = Array.new )

		@target_files = make_array( a_target_files )
		@dependencies = make_array( a_dependencies )
		@build_cmds = make_array( a_build_cmds )
		@clean_cmds = make_array( a_clean_cmds )
	end

	#    .
	#
	# .  _target_ .
	def	build( target )
		need_build = false

		#    .    -
		#    ,    .
		index = 0
		while !need_build && index != @target_files.size
			if Target_state::EXISTS != Target_state.detect(
				@target_files[ index ], @dependencies ).state
				need_build = true
			else
				index += 1
			end
		end

		if need_build
			#       .
			Abstract_target.run( @build_cmds, [] )
		end
	end

	#    .
	#
	# .  _target_ .
	def	clean( target )
		#      ,   
		#   .    
		#      
		#  .
		if 0 != @clean_cmds.size
			@clean_cmds.each do |c|
				system( c )
			end
		else
			@target_files.each do |f|
				Util.delete_file( f )
			end
		end
	end

protected
	#     ,  
	#  .      , 
	#   .
	def	make_array( a_object )
		if a_object.instance_of? Array
			return a_object
		end

		return [ a_object ]
	end
end

end

