#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require	'mxx_ru/ex'
require	'mxx_ru/abstract_target'

module Mxx_ru

#  ,   unit-test   
#      .
#        
#       
#   .
#
#    ,    ,  
#   unit-test.     
# -,   Mxx_ru::Binary_target.
#   unit-test-     , 
#   -  Mxx_ru::Textfile_unittest_target.
# :
#
#    unit-test-.
#	Mxx_ru::setup_target(
#		Mxx_ru::Cpp::Exe_target.new( "test/pack/prj.rb" ) {
#			...
#		}
#	)
#
#    unit-test-.
#	Mxx_ru::setup_target(
#		Mxx_ru::Textfile_unittest_target.new( 
#			"test/pack/pack.ut.rb",
#			"test/pack/prj.rb" ) {
#
#			launch( "--loops 16 --out test/pack/out/16.txt",
#				[ pair( "test/pack/out/16.txt",
#						"test/pack/out/etalon/16.txt" ) ] )
#
#			launch( "--loops 32 --out test/pack/out/32.txt",
#				[ pair( "test/pack/out/32.txt",
#						"test/pack/out/etalon/32.txt" ) ] )
#		}
#	)
#
# ,   unit-test.
#	Mxx_ru::setup_target(
#		Mxx_ru::Cpp::Composite_target( Mxx_ru::BUILD_ROOT ) {
#			required_prj( "some/project/prj.rb" )
#			required_prj( "test/pack/prj.ut.rb" )
#		}
# )
#
class	Textfile_unittest_target < Abstract_target

	# ,  ,  
	#      .
	class	Must_be_one_target_name_ex < Ex
		# _a_names_
		#   .
		def initialize( a_names )
			super( "Only one name expected in [#{a_names.join(';')}]" )
		end
	end

	# ,  ,  
	#   .
	class	File_mismatch_ex < Build_ex
		# _a_cmdline_
		#    .
		#
		# _a_produced_
		#     .
		#
		# _a_etalon_
		#     .
		#
		# _a_line_num_
		#  ,    .
		#
		# _a_produced_line_
		#    .  nil,  
		#  .
		#
		# _a_etalon_line_
		#    .  nil,   
		# .
		def	initialize( a_cmdline,
			a_produced, a_etalon,
			a_line_num,
			a_produced_line, a_etalon_line )
			a_produced_line = "<EOF>" if !a_produced_line
			a_etalon_line = "<EOF>" if !a_etalon_line

			super( a_cmdline, "Mismatch found in line #{a_line_num}. " +
				"Line in '#{a_produced}' is '#{a_produced_line}'. " +
				"Line in '#{a_etalon}' is '#{a_etalon_line}'" )
		end
	end

	#     .
	class	Pair
		#     .
		attr_reader	:produced
		#   .
		attr_reader	:etalon

		# .
		def	initialize( a_produced, a_etalon )
			@produced = a_produced
			@etalon = a_etalon
		end
	end

	#     .
	class	Launch
		#   .
		#  ,   .
		attr_reader	:cmdline
		#       .
		#   Array  Pair.
		attr_reader	:pairs

		# .
		def	initialize( a_cmdline, a_pairs )
			@cmdline = a_cmdline
			@pairs = a_pairs
		end
	end

	# ,     unit-test .
	attr_reader	:mxx_target_project
	#  ,   build  .
	attr_reader	:mxx_build_state
	#    .
	# Array  Launch.
	attr_reader	:mxx_launches

	# .
	#
	# _a_alias_
	#  .
	#
	# _a_target_project_
	# ,     unit-test .
	def	initialize( a_alias, a_target_project, &block )
		super( a_alias )

		@mxx_build_state = nil

		@mxx_target_project = required_prj( a_target_project )

		@mxx_launches = Array.new

		instance_eval( &block ) if block
	end

	#    ,   .
	#
	# _a_produced_
	#     .
	#
	# _a_etalon_
	#     .
	def	pair( a_produced, a_etalon )
		return Pair.new( a_produced, a_etalon )
	end

	#      .
	#
	# _a_cmdline_
	#   .      nil,
	#    .
	#
	# _a_pairs_
	#      .   Array 
	#   pair() .
	#
	#      ,  
	#    _a_pairs_.   
	#   .
	def	launch( a_cmdline, a_pairs )
		a_cmdline = "" if !a_cmdline
		@mxx_launches << Launch.new( a_cmdline, a_pairs )
	end

	#      .
	#
	#   unit-test    0 ,
	#   .
	def	build
		if !@mxx_build_state
			@mxx_target_project.build

			#   ,   .
			full_names = @mxx_target_project.mxx_full_targets_names
			if 1 != full_names.size
				raise Must_be_one_target_name_ex.new( full_names )
			end

			#   dry-run    .
			if !Mxx_ru::Util::Mode.instance.is_dry_run
				puts "running unit test: #{full_names[0]}..."
				do_all_launches( full_names[ 0 ] )
			end

			@mxx_build_state = Target_state.new( Target_state::REBUILT )
		end

		return @mxx_build_state
	end

	#  .
	#
	#   clean   .
	#   ,     unit-test
	# .
	def	clean
		@mxx_target_project.clean

		@mxx_launches.each { |launch|
			launch.pairs.each { |pair|
				Util.delete_file( pair.produced )
			}
		}
	end

protected
	#      .
	#        dry-run, ..
	#        .
	def	do_all_launches( a_file_to_exec )
		#   mswin32     ,
		#     unix-,   
		#    /.
		native_file_name = Util::native_pathname( a_file_to_exec )

		@mxx_launches.each { |launch|
			cmdline = native_file_name + " " + launch.cmdline	
			puts "\tlaunching '#{cmdline}'..."
			if !system( cmdline )
				puts "\n\nunit test '#{cmdline}' FAILED! #{$?}"
				raise Build_ex.new( cmdline, $? )
			end

			#    .
			#    .
			launch.pairs.each { |pair|
				puts "\t\tcomparing '#{pair.produced}' and '#{pair.etalon}'"
				do_file_compare( cmdline, pair.produced, pair.etalon )
			}
		}
	end

	#    .
	#      File_mismatch_ex.
	#   -   (,
	#  ),   Build_ex.
	def	do_file_compare( a_cmdline, a_produced, a_etalon )
		begin
			produced_file = File.new( a_produced, "r" )
			etalon_file = File.new( a_etalon, "r" )

			line_num = 1
			while true
				l1 = produced_file.gets
				l2 = etalon_file.gets

				if l1 != l2
					raise File_mismatch_ex.new( a_cmdline,
						a_produced, a_etalon, line_num,
						l1, l2 )
				else
					line_num += 1
					if !l1
						break
					end
				end
			end

		rescue Exception => ex
			raise Build_ex.new( a_cmdline,
				"Error during comparing files " +
				"'#{a_produced}', '#{a_etalon}': #{ex}" )
		end
	end
end

end # module Mxx_ru
