#
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004, JSC Intervale
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#	this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#	this list of conditions and the following disclaimer in the documentation
#	and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#	from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

require 'rbconfig'
require 'singleton'

module Mxx_ru

	module Util
		#          .
		MATCH_FILE_EXT_REGEX = Regexp.new( "^(.+)\\.([^\\./]+)$" )

		#     Ruby,   Mxx_ru.
		MXXARG_CLEAN = "--mxx-clean"
		MXXARG_SHOW_CMD = "--mxx-show-cmd"
		MXXARG_KEEP_TMPS = "--mxx-keep-tmps"
		MXXARG_SHOW_TMPS = "--mxx-show-tmps"
		MXXARG_DRY_RUN = "--mxx-dry-run"

		#  ,    .
		@@host_os = nil

		# ,  ,     Mxx_ru.
		class	Mode
			include Singleton

			attr_reader	:is_clean
			attr_reader	:is_show_cmd
			attr_reader	:is_keep_tmps
			attr_reader	:is_show_tmps

			#     
			#   .
			def	initialize
				@is_clean = ARGV.include?( MXXARG_CLEAN )
				@is_show_cmd = ARGV.include?( MXXARG_SHOW_CMD )
				@is_keep_tmps = ARGV.include?( MXXARG_KEEP_TMPS )
				@is_show_tmps = ARGV.include?( MXXARG_SHOW_TMPS )
				@is_dry_run = ARGV.include?( MXXARG_DRY_RUN )
				@is_manual_dry_run = false
			end

			#    dry-run?
			#  dry-run ,    
			#  --mxx-dry-run,   dry-run  .
			def	is_dry_run
				return ( @is_dry_run || @is_manual_dry_run )
			end

			# /   dry-run.
			#      -  
			#  dry-run,    . , 
			#      
			# Mxx_ru::Cpp::Target    --mxx-cpp-1.

			def	manual_dry_run( a_is_manual_dry_run )
				@is_manual_dry_run = a_is_manual_dry_run
			end

		end

		#      .
		def Util.remove_file_ext( a_name )
			match_result = MATCH_FILE_EXT_REGEX.match( a_name )
			if nil == match_result
				#    .
				return String.new( a_name )
			end

			return match_result[ 1 ]
		end

		#     .
		def	Util.delete_file( a_name )
			if FileTest.exists?( a_name )
				File.delete( a_name )
			end
		end

		# ,      
		#  .
		#      .
		#     
		# ,     --mxx-keep-tmps
		class	Tmp_files
			include Singleton

			#   .
			@@names = Array.new

			def	initialize
				if !Mode.instance.is_keep_tmps
					ObjectSpace.define_finalizer( self, Proc.new{ Tmp_files.finalizer } )
				end

				#      .
				@current = 0

				#    @@names    push, pop.
				@index_stack = Array.new
			end

			def	Tmp_files.finalizer
				@@names.each { |d|
						begin
							File.delete( d )
						rescue Exception => e
							puts e
						end
					}
			end

			#   .
			#      .
			#    .
			def	create(
				#     String.
				a_content )

				current = (@current += 1)
				file_name = "tmpmxx_ru.#{$$}.#{current}"
				file = File.new( file_name, "w" )
				@@names << file_name
				file << a_content
				file.close

				if Mode.instance.is_show_tmps
					puts "<<<[#{file_name}]\t #{a_content}>>>"
				end

				return file_name
			end

			#       .
			def	push()
				if !Mode.instance.is_keep_tmps
					@index_stack.push( @@names.size() )
				end
			end

			#   ,      
			#   ,     
			#  @index_stack.
			def	pop()
				if !Mode.instance.is_keep_tmps
					index = @index_stack.pop
					while @@names.size > index
						file_to_delete = @@names.pop
						Mxx_ru::Util::delete_file( file_to_delete )
					end
				end
			end

		end # class Tmp_files

		#       build
		#   target.
		#    build  Tmp_files.push,  
		#   build  Tmp_files.pop.   
		#  ,     .
		def	Util.build_call_wrapper( a_target )
			begin
				Tmp_files.instance.push
				state = a_target.build
				return state
			ensure
				Tmp_files.instance.pop
			end
		end

		#  ,   .
		#
		#    . ,  
		#   output/release/lib,    
		# output,    release  release/lib.
		def	Util.create_dir( a_name )
			#     .
			chain = Array.new
			chain.unshift( a_name )

			to_split = a_name.clone
			while to_split != "."
				r = File.split( to_split )
				chain.unshift( to_split = r[ 0 ] )
			end

			chain.each { |d|
				if !FileTest.exists?( d )
					Dir.mkdir( d )
				end
			}
		end

		# ,   .
		#
		#     dry-run,   
		#  .    ,  
		#  .
		#
		#   dry-run  ,   ,  
		#      .
		def	Util.ensure_path_exists( a_path_name )
			if !Mxx_ru::Util::Mode.instance.is_dry_run
				if !FileTest.exists?( a_path_name )
					Mxx_ru::Util.create_dir( a_path_name )
				end
			end
		end

		#   ,    .
		def	Util.host_os
			if !@@host_os
				@@host_os = Config::CONFIG[ "host_os" ]
			end

			return @@host_os
		end

		#       ,
		#     .
		#
		# ,   mswin32   
		#  .
		#
		# _a_pathname_
		#     UNIX-.
		#
		# *!       mswin32!*
		def Util.native_pathname( a_pathname )
			if "mswin32" == Util::host_os
				return a_pathname.gsub( "/", "\\" )
			end

			return a_pathname
		end

	end # module Util

end # module Mxx_ru
